package Principal;

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.events.Characters;
import javax.xml.stream.events.XMLEvent;
import Modelo.FeedMessage;

/**
 * Clase donde se realiza la lectura del archivo XML que se encuentra en el
 * RSS proporcionado.
 * @author Adrián
 */
public class RSSFeedParser {
    //Nombre de las etiquetas del archivo XML, cuyo contenido se almacenará.
    static final String TITLE = "title";
    static final String LINK = "link";
    static final String ITEM = "item";
    static final String PUB_DATE = "pubDate";
    //URL  a utilizar.
    final URL url;
    /**
     * Recibe la URL para leer, y actualiza la variable con su valor.
     * @param feedUrl 
     */
    public RSSFeedParser(String feedUrl) {
        try {
            this.url = new URL(feedUrl);
        } catch (MalformedURLException e) {
            throw new RuntimeException(e);
        }
    }
    /**
     * Método que realiza la lectura del archivo XML.
     * @return Devuelve una instancia de la clase Feed que contiene un ArrayList con
     * los titulares leidos.
     */
    public Feed readFeed() {
        Feed feed = null;
        try {
            boolean isFeedHeader = true;
            // Variables para almacenar el contenido de la etiqueta.
            String title = "";
            String link = "";
            String pubdate = "";
            //Lectura del RSS.
            XMLInputFactory inputFactory = XMLInputFactory.newInstance();
            InputStream in = read();
            //Si es null es que hay algún problema de conexión al leer la URL.
            //Si hay problemas, se pasa a la siguiente URL.
            if(in!=null){
                XMLEventReader eventReader = inputFactory.createXMLEventReader(in);
                //Lectura del archivo XML.
                while (eventReader.hasNext()) {
                    XMLEvent event = eventReader.nextEvent();
                    if (event.isStartElement()) {
                        String localPart = event.asStartElement().getName().getLocalPart();
                        //Almacenamiento del contenido para cada etiqueta leida.
                        switch (localPart) {
                            //Cada titular se encuentra dentro de la etiqueta item.
                            case ITEM:
                                if (isFeedHeader) {
                                    isFeedHeader = false;
                                    feed = new Feed(link, pubdate);
                                }
                                event = eventReader.nextEvent();
                            break;
                            //Almacenamiento del titular.
                            case TITLE:
                                title = getCharacterData(event, eventReader);
                                int longit= title.length();
                                //Para evitar titulares vacíos, subidos por error, por el periódico.
                                if(longit>0){
                                    char ult =title.charAt(longit-1);
                                    if(ult==' '){
                                        title=title.substring(0, longit-1);                        
                                    }
                                }
                            break;
                            //Almacenamiento de la URL.
                            case LINK:
                                link = getCharacterData(event, eventReader);
                            break;
                            //Almacenamiento de la fecha de publicación del titular.
                            case PUB_DATE:
                                pubdate = getCharacterData(event, eventReader);
                            break;
                        }
                    } else if (event.isEndElement()) {
                        if (event.asEndElement().getName().getLocalPart() == (ITEM)) {
                            //Creación de una intancia FeedMessage que se corresponde con un titular y su información.
                            FeedMessage message = new FeedMessage();
                            //Comproabación de que la fecha es correcta, es decir, formado por el número de campos correcto.
                            String[] partes_fecha = pubdate.split(" ");
                            //Almacenamiento de los datos leidos si la comprobación es satisfactoria, sino, se ignora el titular.
                            if(title.length()>0 && pubdate.split(" ").length==6 && partes_fecha[4].split(":").length==3){
                                message.setTitle(title);
                                message.setCampo_fecha(pubdate);
                                feed.getMessages().add(message);
                            }
                        event = eventReader.nextEvent();
                        continue;
                        }
                    }
                }
            }
        } catch (XMLStreamException e) {
            throw new RuntimeException(e);
        }
        return feed;
    }
    /**
     * Obtención de un String con el contenido correspondiente, de cada etiqueta.
     * @param event
     * @param eventReader
     * @return Devuelve la cadena de caracteres leida.
     * @throws XMLStreamException 
     */
    private String getCharacterData(XMLEvent event, XMLEventReader eventReader)
            throws XMLStreamException {
                String result = "";
                event = eventReader.nextEvent();
                if (event instanceof Characters) {
                    result = event.asCharacters().getData();
                }
        return result;
    }
    /**
     * Método para conectarse con la URL.
     * Si no es posible conectarse con la URL, se imprime un mesaje en consola.
     * @return Devuelve la lectura de la URL.
     */
    private InputStream read() {
        try {
            return url.openStream();
        } catch (IOException e) {
            System.out.println("Error accediendo a: "+url);
            return null;
        }
    }
}